from argparse import ArgumentParser
from collections.abc import Sequence


class AppContext:
    def __init__(self, **kwargs):
        self.command: str | None = None

        self.max_workers: int = 4
        self.profile_id: str | None = None
        self.search_pattern: str | None = None
        self.save_target: str | None = None

        self.show_full_help: bool = False
        self.show_help = False

        self.debug_mode: bool = False
        self.pretending_mode: bool = False

        for k, v in kwargs.items():
            if k in self.__dict__:
                self.__dict__[k] = v

    def __rich_repr__(self):
        yield from self.__dict__.items()

    @classmethod
    def from_arguments(cls, args: Sequence[str]) -> "AppContext":
        global_parser = cls.__global_parser()
        main_parser = cls.__command_parser()

        global_args, remaining_args = global_parser.parse_known_args(args)

        cmd_args = main_parser.parse_args(remaining_args)

        for key, value in vars(global_args).items():
            setattr(cmd_args, key, value)

        return cls(**vars(cmd_args))

    @staticmethod
    def __global_parser() -> ArgumentParser:
        parser = ArgumentParser(add_help=False)
        parser.add_argument(
            "-h", "--help", action="store_true", dest="show_help", help="显示此帮助信息"
        )
        parser.add_argument(
            "--tutorial",
            "--full-help",
            action="store_true",
            dest="show_full_help",
            help="显示详细教程",
        )
        parser.add_argument(
            "--debug", "-d", action="store_true", dest="debug_mode", help="开启调试模式"
        )
        parser.add_argument(
            "--pretend",
            "-p",
            action="store_true",
            dest="pretending_mode",
            help="假装执行，不实际更新 hosts 文件",
        )
        return parser

    @staticmethod
    def __command_parser() -> ArgumentParser:
        main_parser = ArgumentParser(add_help=False)
        subparsers = main_parser.add_subparsers(
            dest="command", required=False, help="子命令"
        )

        update_parser = subparsers.add_parser(
            "update", help="更新 hosts 文件", description="更新 hosts 文件"
        )
        update_parser.add_argument(
            "--target",
            "--to",
            "-t",
            help="指定 hosts 文件路径",
            dest="save_target",
            required=False,
            type=str,
            default=None,
        )

        list_parser = subparsers.add_parser(
            "list", help="列出所有配置文件", description="列出所有配置文件"
        )
        list_parser.add_argument(
            "--search",
            "-s",
            help="搜索模式",
            dest="search_pattern",
            required=False,
            type=str,
            default="",
        )

        show_parser = subparsers.add_parser(
            "show", help="显示指定配置文件内容", description="显示指定配置文件内容"
        )
        show_parser.add_argument(
            "profile_id", help="配置文件 ID", type=str, metavar="配置文件 ID"
        )

        new_parser = subparsers.add_parser(
            "new", help="创建新配置文件", description="创建新配置文件"
        )
        new_parser.add_argument(
            "profile_id", help="配置文件 ID", type=str, metavar="配置文件 ID"
        )

        edit_parser = subparsers.add_parser(
            "edit", help="编辑指定配置文件", description="编辑指定配置文件"
        )
        edit_parser.add_argument(
            "profile_id", help="配置文件 ID", type=str, metavar="配置文件 ID"
        )

        return main_parser



from cx_studio.core import FileSize
from pathlib import Path
from collections.abc import Iterable


class FileSizeCounter:
    def __init__(self, paths: Iterable[Path | str] | None = None) -> None:
        self._paths = [Path(x) for x in paths or []]

    def add_path(self, path: Path | str):
        self._paths.append(Path(path))

    def add_paths(self, paths: Iterable[Path | str]):
        self._paths.extend([Path(x) for x in paths])

    @property
    def total_size(self) -> FileSize:
        if not self._paths:
            return FileSize.from_bytes(0)
        sizes = sum([x.stat().st_size if x.exists() else 0 for x in self._paths])
        return FileSize.from_bytes(sizes)



from typing import override

from hosts_keeper.profile.contenter_base import ContenterBase
from media_scout.inspectors import edl_inspector

from .profile_manager import ProfileManager
from .appenv import appenv
from collections.abc import Iterable, Sequence
import sys
from cx_tools.app import IApplication
from .hosts_builder import HostsBuilder
from cx_wealth import rich_types as r
from cx_wealth import WealthDetailPanel, IndexedListPanel, WealthLabel
import os, subprocess
from .hosts_saver import HostsSaver
from cx_studio.utils import SystemUtils
from pathlib import Path
from .app_help import AppHelp


class Application(IApplication):
    def __init__(self, arguments: Sequence[str] | None = None):
        super().__init__(arguments or sys.argv[1:])
        self.profile_manager = ProfileManager()

    @override
    def start(self):
        appenv.load_arguments(self.sys_arguments)
        appenv.start()
        if appenv.is_debug_mode_on():
            appenv.say("[cx.warning]调试模式已开启。")
            appenv.whisper(
                IndexedListPanel(
                    [WealthLabel(x) for x in self.profile_manager.profiles.values()],
                    title="已找到配置文件",
                )
            )
        return self

    @override
    def stop(self):
        appenv.stop()
        return self

    @staticmethod
    def __open_file(file_path: Path):
        editor = os.environ.get("EDITOR", None)
        if editor:
            subprocess.run(f"{editor} {filename}")
            return

        appenv.whisper(f"[cx.warning]未设置编辑器环境变量，尝试使用系统工具打开。")
        result = SystemUtils.open(file_path)
        if not result:
            url = f"file://{file_path.resolve()}"
            appenv.say(f"[cx.error]打开文件 [link={url}]{file_path.name}[/link] 失败。")

    @staticmethod
    def __open_dir(dir_path: Path):
        result = SystemUtils.open(dir_path)
        if not result:
            url = f"file://{dir_path.resolve()}"
            appenv.say(f"[cx.error]打开目录 [link={url}]{dir_path.name}[/link] 失败。")

    def command_new(self):
        profile_id = appenv.context.profile_id
        filename = self.profile_manager.generate_profile_path(profile_id)
        if filename.exists():
            appenv.say(f"[cx.error]配置文件 {filename.name} 已存在。")
            return

        filename = self.profile_manager.create_profile(profile_id, filename)
        if filename:
            appenv.say(
                f"[cx.success]已创建配置文件: [link=file://{filename.resolve()}]{filename.name}[/link]"
            )
        else:
            appenv.say(f"[cx.error]创建配置文件失败。")
            return

        self.__open_file(filename)

    def command_list(self):
        table = r.Table(
            r.Column("ID", highlight=False, style="yellow"),
            r.Column("Name", highlight=False, style="cyan"),
            r.Column("Description", highlight=False, style="green"),
            r.Column("Enabled", highlight=False),
            box=r.box.HORIZONTALS,
            border_style="dim blue",
            header_style="bold blue",
        )
        for profile in self.profile_manager.find_profile(appenv.context.search_pattern):
            table.add_row(
                profile.id,
                profile.name,
                profile.description,
                "[cx.success]YES[/]" if profile.enabled else "[cx.error]NO[/]",
            )
        if table.row_count == 0:
            appenv.say("[cx.warning]未找到符合条件的配置文件。")
        else:
            appenv.say(table)
            appenv.say(f"[cx.success]共找到 {table.row_count} 个配置文件。")
            appenv.say(f"[dim]可尝试使用 show 或 edit 命令查看或编辑配置文件。")

    def command_show(self):
        profile = self.profile_manager.profiles.get(appenv.context.profile_id, None)
        if profile:
            appenv.say(WealthDetailPanel(profile, title=profile.id))
        else:
            appenv.say(
                f"[cx.error]未找到 ID 为 {appenv.context.profile_id} 的配置文件。"
            )

    def command_edit(self):
        profile_id = appenv.context.profile_id
        profile = self.profile_manager.profiles.get(profile_id, None)
        file_path = (
            profile.path.resolve() if profile else appenv.config_manager.config_dir
        )

        if not file_path or not file_path.exists():
            appenv.say(f"[cx.error]未找到 ID 为 {profile_id} 的配置文件。")
            return

        if file_path.is_dir():
            self.__open_dir(file_path)
        else:
            self.__open_file(file_path)

    def command_update(self):
        appenv.whisper("update 模式已启动")
        builder = HostsBuilder()
        appenv.whisper("已构建 HostBuilder")

        enabled_profiles = list(self.profile_manager.enabled_profiles)
        appenv.whisper(
            IndexedListPanel(
                [WealthLabel(x) for x in enabled_profiles], title="已启用配置文件"
            )
        )

        appenv.whisper("开始构建 Hosts 文件内容")
        lines = builder.iter_lines(enabled_profiles)

        # 保存临时文件
        with appenv.temp_hosts.open("w", encoding="utf-8") as f:
            for line in lines:
                f.write(line + "\n")
        appenv.whisper(f"已写入新的内容到临时文件 {appenv.temp_hosts}")

        saver = HostsSaver()
        save_target = None
        if appenv.context.save_target:
            save_target = Path(appenv.context.save_target)
            appenv.whisper(f"将保存到目标文件 {save_target}")
        saved = saver.save(save_target)
        if saved:
            appenv.say(f"[cx.success]已成功保存新的 hosts 文件。")
            self.show_refresh_tips()

    def show_refresh_tips(self):
        if sys.platform.startswith("win"):
            appenv.say(
                "[cx.info]请在管理员权限下执行 ipconfig /flushdns 以刷新 DNS 缓存。"
            )
        else:
            appenv.say("[cx.info]请别忘了刷新DNS缓存以应用新的 hosts 文件。")

    def command_help(self):
        AppHelp.show_help(appenv.console)

    def run(self):
        if appenv.context.command == "help" or appenv.context.show_help:
            self.command_help()
            return

        if appenv.context.show_full_help:
            AppHelp.show_full_help(appenv.console)
            return

        if appenv.context.command == "new":
            self.command_new()
            return

        if appenv.context.command == "list":
            self.command_list()
            return

        if appenv.context.command == "show":
            self.command_show()
            return

        if appenv.context.command == "edit":
            self.command_edit()
            return

        if appenv.context.command == "update":
            self.command_update()
            return

        self.command_help()



import asyncio
import importlib
import tomllib
from pathlib import Path
from box import Box, BoxList

# from pydantic import BaseModel, Field, ConfigDict
from typing import AsyncGenerator, Self, Sequence
from .hostrecord import HostRecord
from dataclasses import dataclass, field
from cx_studio.utils import PathUtils

from .contenter_base import ContenterBase


@dataclass(frozen=True)
class Profile:
    # model_config = ConfigDict(frozen=True)

    id: str = ""
    name: str = ""
    description: str = ""
    priority: int = 0
    enabled: bool = True
    path: Path = field(default_factory=Path)

    metadata: Box = field(default_factory=Box)
    packages: Box = field(default_factory=Box)

    @classmethod
    def load(cls, filename: Path | str) -> Self | None:
        filename = PathUtils.force_suffix(filename, ".toml")
        with open(filename, "rb") as f:
            toml = tomllib.load(f)
        data = Box(toml)

        metadata = data.get("hosts_profile")
        if not metadata:
            return None

        packages_data = data.copy()
        packages_data.pop("hosts_profile")

        return cls(
            id=metadata.profile_id,  # type:ignore
            name=metadata.profile_name,  # type:ignore
            description=metadata.description,  # type:ignore
            path=Path(filename).resolve(),
            priority=metadata.priority,  # type:ignore
            enabled=metadata.enabled,  # type:ignore
            metadata=metadata,  # type:ignore
            packages=packages_data,  # type:ignore
        )

    @staticmethod
    def create(profile_id: str, target: Path) -> Path:
        target = PathUtils.force_suffix(target, ".toml")
        target.parent.mkdir(parents=True, exist_ok=True)

        example = importlib.resources.read_text(__package__, "example_profile.toml")
        example = example.replace("example-id", profile_id)

        with open(target, "w", encoding="utf-8") as f:
            f.write(example)

        return target

    @property
    def profile_start_marker(self) -> str:
        """配置文件开始标记"""
        return f"##### {self.id} START #####"

    PROFILE_START_MARKER_PATTERN = r"#####\s+(.+)\s+START\s+#####"

    @property
    def profile_end_marker(self) -> str:
        """配置文件结束标记"""
        return f"##### {self.id} END #####"

    PROFILE_END_MARKER_PATTERN = r"#####\s+(.+)\s+END\s+#####"

    async def async_iter_records(self) -> AsyncGenerator[HostRecord, None]:
        """迭代记录"""

        async def expand_contenter(contenter: ContenterBase):
            result = []
            async for record in contenter.iter_records():
                result.append(record)
            return result

        tasks = []

        for schema, packages in self.packages.items():
            if not isinstance(packages, list | BoxList):
                packages = [packages]
            for package in packages:
                contenter = ContenterBase.create_contenter(
                    schema, package, self.metadata
                )
                if contenter is None:
                    continue
                tasks.append(asyncio.create_task(expand_contenter(contenter)))

        async for task in asyncio.as_completed(tasks):
            result = await task
            for record in result:
                yield record

    async def async_iter_lines(
        self, include_markers: bool = True
    ) -> AsyncGenerator[str, None]:
        """迭代行"""
        if include_markers:
            yield ""
            yield self.profile_start_marker
        async for record in self.async_iter_records():
            yield str(record)
        if include_markers:
            yield self.profile_end_marker
            yield ""

    def __rich_label__(self) -> str:
        enabled = "✅" if self.enabled else "❌"
        yield enabled
        yield self.id

    def __rich_repr__(self):
        # yield "id", self.id
        yield "name", self.name
        yield "description", self.description
        yield "priority", self.priority
        yield "enabled", self.enabled
        yield "path", self.path
        for k, v in self.packages.items():
            if isinstance(v, Sequence):
                for i in range(len(v)):
                    yield f"{k} {i}", v[i]
            else:
                yield k, v



from .application import Application


def run():
    from rich.traceback import install

    install(show_locals=False, word_wrap=True, suppress=["rich"])

    with Application() as app:
        app.run()



from argparse import ArgumentParser
from dataclasses import dataclass
from collections.abc import Sequence
from typing import Literal
from xml.etree.ElementInclude import include

from cx_studio.utils.cx_textutils import auto_quote
from cx_wealth import WealthHelp
from cx_wealth import rich_types as r
import importlib.resources


class ArgParser(ArgumentParser):
    def __init__(self):
        super().__init__(
            prog="MediaScout",
            description="MediaScout is a tool for inspecting media files.",
            epilog="",
            add_help=False,
        )

        self.add_argument("inputs", nargs="*", metavar="INPUTS")
        self.add_argument("-i", "--include", dest="includes", nargs="*")
        self.add_argument(
            "-e",
            "--existed-only",
            dest="existed_only",
            default=False,
            action="store_true",
        )
        self.add_argument(
            "-o",
            "--output",
            action="store",
            dest="output",
            metavar="OUTPUT",
            default=None,
        )
        self.add_argument(
            "--allow-duplicated",
            action="store_true",
            default=False,
            dest="allow_duplicated",
        )
        self.add_argument(
            "-q",
            "--quote-mode",
            default="none",
            choices=["auto", "force", "escape", "none"],
            dest="quote_mode",
        )
        self.add_argument(
            "--auto-resolve", action="store_true", default=False, dest="auto_resolve"
        )
        self.add_argument(
            "-d", "--debug", action="store_true", default=False, dest="debug_mode"
        )
        self.add_argument(
            "-h", "--help", action="store_true", default=False, dest="show_help"
        )
        self.add_argument(
            "--tutorial",
            "--full-help",
            action="store_true",
            default=False,
            dest="show_full_help",
        )


@dataclass
class AppContext:
    inputs: list[str]
    includes: list[str]
    output: str | None
    allow_duplicated: bool
    auto_resolve: bool
    existed_only: bool
    quote_mode: Literal["auto", "force", "escape", "none"]
    debug_mode: bool
    show_help: bool
    show_full_help: bool

    @classmethod
    def load(cls, arguments: Sequence[str] | None = None):
        parser = ArgParser()
        args = parser.parse_args(arguments)
        return cls(
            inputs=args.inputs or [],
            includes=args.includes or [],
            output=args.output,
            allow_duplicated=args.allow_duplicated,
            auto_resolve=args.auto_resolve,
            existed_only=args.existed_only,
            quote_mode=args.quote_mode,
            debug_mode=args.debug_mode,
            show_help=args.show_help,
            show_full_help=args.show_full_help,
        )

    def __rich_detail__(self):
        yield from self.__dict__.items()


class MSHelp(WealthHelp):
    def __init__(self):
        super().__init__(
            prog="MediaScout",
            description=(
                "解析时间线、元数据表格等项目文件，从中提取包含的文件路径。支持干净的输出流或输出到文件。"
                "目前可以解析的类型包括：EDL表、FCP7 经典XML、FCP XML文件或包、Davinci Resolve 媒体池元数据、纯文本路径列表等。"
            ),
        )
        p_group = self.add_group("文件输入", "输入需要解析的文件，可以一次解析多个。")
        p_group.add_action(
            "inputs", metavar="FILE", nargs="*", description="需要解析的文件"
        )

        o_group = self.add_group("选项", "对结果进行处理的若干选项")
        o_group.add_action(
            "-i",
            "--include",
            metavar="DIR",
            nargs="**",
            description="指定用于搜索无路径文件名的文件夹",
        )
        o_group.add_action("-e", "--existed-only", description="仅输出已存在的文件路径")
        o_group.add_action("--allow-duplicated", description="允许重复输出文件路径")
        o_group.add_action("--auto-resolve", description="自动解析并整理文件路径")
        o_group.add_action(
            "-q",
            "--quote-mode",
            metavar="auto|force|excape|none",
            description="指定对于包含空格的路径的处理方式",
        )
        o_group.add_action(
            "-o", "--output", metavar="OUTPUT", description="将文件列表保存到目标文件"
        )

        x_group = self.add_group("其它")
        x_group.add_action("-d", "--debug", description="开启调试模式")
        x_group.add_action("-h", "--help", description="显示帮助信息")
        x_group.add_action(
            "--tutorial", "--full-help", description="显示完整的帮助信息"
        )

        self.epilog = (
            "[link https://github.com/LambdaXIII/cx-studio-tk]Cxalio Studio Tools[/]"
        )

    @staticmethod
    def show_help(console: r.Console):
        console.print(MSHelp())

    @staticmethod
    def show_full_help(console: r.Console):
        md = importlib.resources.read_text("media_scout", "help.md")
        content = r.Markdown(md, style="default")
        panel = r.Panel(
            content,
            title="MediaScout 教程",
            title_align="left",
            style="bright_black",
            width=90,
        )
        console.print(r.Align.center(panel))



from collections.abc import Iterable, Generator
import xml.etree.ElementTree as ET


import ulid
from .mission import Mission
from pathlib import Path
from .argument_group import ArgumentGroup
from cx_studio.utils import PathUtils


class MissionXML:
    def __init__(self) -> None:
        self.root = ET.Element("missions")

    @staticmethod
    def _encode_argument_group(
        group: ArgumentGroup, name: str | None = None
    ) -> ET.Element:
        node = ET.Element(name or "argument_group")
        if group.filename:
            node.set("filename", str(group.filename))

        arguments = list(group.iter_arguments())
        if arguments:
            node.text = " ".join(arguments)

        return node

    @staticmethod
    def encode_mission_node(mission: Mission):
        mission_node = ET.Element("mission")
        mission_node.set("mission_id", str(mission.mission_id))
        mission_node.set("preset_id", mission.preset_id)
        mission_node.set("preset_name", mission.preset_name)

        mission_node.set("source", str(mission.source))
        mission_node.set("standard_target", str(mission.standard_target))

        ffmpeg_node = ET.SubElement(mission_node, "ffmpeg")
        ffmpeg_node.text = mission.ffmpeg

        # source_node = ET.SubElement(mission_node, "source")
        # source_node.text = str(mission.source)

        # std_target_node = ET.SubElement(mission_node, "standard_target")
        # std_target_node.text = str(mission.standard_target)

        overwrite_node = ET.SubElement(mission_node, "overwrite")
        overwrite_node.text = "YES" if mission.overwrite else "NO"

        hwaccel_node = ET.SubElement(mission_node, "hardware_accelerate")
        hwaccel_node.text = mission.hardware_accelerate

        options_node = MissionXML._encode_argument_group(mission.options, "options")
        mission_node.append(options_node)

        inputs_node = ET.SubElement(mission_node, "inputs")
        for input_group in mission.inputs:
            inputs_node.append(MissionXML._encode_argument_group(input_group))

        output_node = ET.SubElement(mission_node, "outputs")
        for output_group in mission.outputs:
            output_node.append(MissionXML._encode_argument_group(output_group))

        return mission_node

    def add_mission(self, mission: Mission):
        mission_node = MissionXML.encode_mission_node(mission)
        self.root.append(mission_node)

    def add_missions(self, missions: Iterable[Mission]):
        for mission in missions:
            self.add_mission(mission)

    def __len__(self):
        return len(self.root.findall("mission"))

    @staticmethod
    def _decode_argument_group(node: ET.Element) -> ArgumentGroup:
        filename = node.get("filename")
        args = node.text.split(" ") if node.text else []
        return ArgumentGroup(
            options=args, filename=Path(filename) if filename else None
        )

    @staticmethod
    def decode_mission_node(node: ET.Element) -> Mission:
        mission_id = ulid.from_str(str(node.get("mission_id")))
        preset_id = str(node.get("preset_id"))
        preset_name = str(node.get("preset_name"))
        source = str(node.get("source"))
        standard_target = str(node.get("standard_target"))

        def get_subnode_text(name: str) -> str | None:
            subnode = node.find(name)
            return subnode.text if subnode else None

        ffmpeg = get_subnode_text("ffmpeg")

        overwrite = get_subnode_text("overwrite") == "YES"
        hardware_accelerate = get_subnode_text("hardware_accelerate")

        options_node = node.find("options")
        options = (
            MissionXML._decode_argument_group(options_node) if options_node else None
        )

        inputs = []
        inputs_node = node.find("inputs")
        if inputs_node:
            for input_node in inputs_node.findall("argument_group"):
                inputs.append(MissionXML._decode_argument_group(input_node))

        outputs = []
        outputs_node = node.find("outputs")
        if outputs_node:
            for output_node in outputs_node.findall("argument_group"):
                outputs.append(MissionXML._decode_argument_group(output_node))

        return Mission(
            preset_id=preset_id,
            preset_name=preset_name,
            ffmpeg=ffmpeg or "ffmpeg",
            source=Path(source or ""),
            standard_target=Path(standard_target or ""),
            overwrite=overwrite,
            hardware_accelerate=hardware_accelerate or "auto",
            options=options or ArgumentGroup(),
            inputs=inputs,
            outputs=outputs,
            mission_id=mission_id,
        )

    def iter_missions(self) -> Generator[Mission, None, None]:
        for mission_node in self.root.findall("mission"):
            yield MissionXML.decode_mission_node(mission_node)

    def clear(self):
        self.root.clear()

    def save(self, path: Path):
        tree = ET.ElementTree(self.root)
        path = PathUtils.ensure_parents(path)
        tree.write(path, encoding="utf-8", xml_declaration=True)

    @classmethod
    def load(cls, path: Path):
        tree = ET.parse(path)
        root = tree.getroot()
        result = cls()
        result.root = root
        return result



from pathlib import PurePath
import re
from typing import Iterable
import xml.etree.ElementTree as ET


from .media_path_inspector import MediaPathInspector
from .inspector_info import InspectorInfo

import urllib.parse


class LegacyXMLInspector(MediaPathInspector):
    DOCTYPE_PATTERN = re.compile(r"<!DOCTYPE\s+xmeml>")
    URL_HEAD = re.compile(r"^file://.*?/")

    def _is_inspectable(self, info: InspectorInfo) -> bool:
        if info.path.suffix != ".xml":
            return False
        if not info.is_decodable():
            return False
        for line in info.peek_lines():
            if self.DOCTYPE_PATTERN.search(line):
                return True
        return False

    def _inspect(self, info: InspectorInfo) -> Iterable[PurePath]:
        tree = ET.parse(info.path)
        root = tree.getroot()
        for node in root.iter("pathurl"):
            url = urllib.parse.unquote(node.text or "")
            if self.URL_HEAD.match(url):
                path = self.URL_HEAD.sub("", url)
                yield PurePath(path)



from dataclasses import dataclass, field


@dataclass
class HostRecord:
    ip: str | None = None
    domains: list[str] = field(default_factory=list)
    comment: str | None = None

    def is_pure_comment(self) -> bool:
        """
        判断是否为注释行。
        """
        return (self.ip is None) and (self.comment is not None)

    def is_valid(self) -> bool:
        """
        判断是否为有效记录。
        """
        is_useful = (self.ip is not None) and self.domains
        return is_useful or self.is_pure_comment()

    @classmethod
    def from_line(cls, line: str) -> "HostRecord":
        """
        从 hosts 内容行创建 HostRecord 对象。
        """
        if not line:
            return cls()

        if line.startswith("#"):
            return cls(comment=line.removeprefix("#").strip())

        # TODO: 使用regex精细化解析
        ip, *domains = line.split()
        return cls(ip, domains)

    @classmethod
    def from_comment(cls, comment: str) -> "HostRecord":
        """
        创建注释记录。
        """
        return cls(comment=comment)

    def __str__(self) -> str:
        """
        将 HostRecord 对象转换为 hosts 内容行。
        """
        if not self.is_valid():
            return ""
        if self.is_pure_comment():
            return f"# {self.comment}"
        result = f"{self.ip}\t{' '.join(self.domains)}"
        if self.comment:
            result += f" #{self.comment}"
        return result



import asyncio
import importlib.resources
import importlib.resources
import sys
from collections.abc import Sequence
from pathlib import Path
from typing import override

from cx_studio.utils import PathUtils
from cx_tools.app import IApplication
from cx_wealth import DynamicColumns, IndexedListPanel, WealthDetailPanel


from .components.mission_master import MissionMaster
from .components.mission_xml import MissionXML
from .components.script_maker import ScriptMaker
from .components.input_scanner import InputScanner
from .components.mission_maker import MissionMaker
from .components.mission_arranger import MissionArranger
from .components.mission import Mission
from .components.preset import Preset
from .components.exception import SafeError

from .appenv import appenv
from .mk_help_info import MKHelp


class Application(IApplication):
    def __init__(self, arguments: Sequence[str] | None = None):
        super().__init__(arguments or sys.argv[1:])
        self.presets: list[Preset] = []
        self.sources: list[Path] = []
        self.missions: list[Mission] = []

    def start(self):
        appenv.load_arguments(self.sys_arguments)
        appenv.start()
        appenv.show_banner()
        return self

    def stop(self):
        if not appenv.context.continue_mode:
            self.save_missions(self.missions)
        appenv.whisper("Bye ~")
        appenv.stop()

    @override
    def __exit__(self, exc_type, exc_val, exc_tb):
        result = super().__exit__(exc_type, exc_val, exc_tb)
        if exc_type is None:
            appenv.whisper("程序正常退出。")
        elif exc_type is SafeError:
            appenv.say(exc_val)
            result = True
        return result

    @staticmethod
    def save_missions(missions: list[Mission]):
        mission_xml = MissionXML()
        mission_xml.add_missions(missions)
        mission_xml.save(appenv.config_manager.get_file("last_missions.xml"))

    @staticmethod
    def load_missions() -> list[Mission]:
        last_missions = appenv.config_manager.get_file("last_missions.xml")
        if not last_missions.exists():
            return []
        mission_xml = MissionXML.load(
            appenv.config_manager.get_file("last_missions.xml")
        )
        return list(mission_xml.iter_missions())

    @staticmethod
    def export_example_preset(filename: Path):
        filename = Path(PathUtils.force_suffix(filename, ".toml"))
        appenv.check_overwritable_file(filename)
        with importlib.resources.open_text(
            "media_killer", "example_preset.toml"
        ) as example:
            with open(filename, "w", encoding="utf-8") as f:
                f.write(example.read())

        appenv.say(
            f"[cx.success]已生成示例配置文件：{filename}。[/][blink red]请在修改后使用！[/]"
        )

    def _set_presets_and_sources(self, presets, sources):
        # 去除重复的配置文件
        preset_ids = set()
        for p in presets:
            if p.id in preset_ids:
                appenv.say(
                    "[cx.warning]发现重复的配置文件: [cx.warning]{}[/]".format(p.path)
                )
                continue
            preset_ids.add(p.id)
            self.presets.append(p)

        appenv.whisper(
            DynamicColumns(WealthDetailPanel(x, title=x.id) for x in self.presets)
        )

        self.sources += list(sources)
        appenv.whisper(IndexedListPanel(self.sources, "来源路径列表"))

        if self.presets or self.sources:
            appenv.say(
                "已添加 {preset_count} 个配置文件和 {source_count} 个来源路径。".format(
                    preset_count=len(self.presets), source_count=len(self.sources)
                )
            )

    def _sort_and_set_missions(self, missions):
        self.missions = list(MissionArranger(missions, appenv.context.sort_mode))
        # 检查任务数量并判断是否运行
        if not self.missions:
            raise SafeError("没有任务需要执行。")
        # 汇报任务数量
        old_count, new_count = len(missions), len(self.missions)
        if old_count != new_count:
            appenv.say(
                "[cx.warning]已自动过滤掉 {} 个重复任务，共 {} 个任务需要执行。[/]".format(
                    old_count - new_count, new_count
                )
            )
        else:
            appenv.say("全部任务整理完毕，已按照设定方式排序。")
        appenv.whisper(IndexedListPanel(self.missions, "整理完的任务列表"))

    def run(self):
        if appenv.context.show_help:
            MKHelp.show_help(appenv.console)
            return

        if appenv.context.show_full_help:
            MKHelp.show_full_help(appenv.console)
            return

        # 是否生成配置文件
        if appenv.context.generate:
            for s in appenv.context.inputs:
                s = Path(s)
                suffix = s.suffix
                if suffix == ".toml" or suffix == "":
                    self.export_example_preset(s)
                else:
                    appenv.whisper(
                        "{filename} 并非合法的文件名，不予处理。".format(filename=s)
                    )
            return

        # 扫描输入文件
        with InputScanner(appenv.context.inputs) as input_scanner:
            presets, sources = input_scanner.scan()

        self._set_presets_and_sources(presets, sources)

        # 恢复上次的任务
        missions = []
        if appenv.context.continue_mode:
            last_missions = self.load_missions()
            appenv.say("从上次执行中恢复了 {} 个任务……".format(len(last_missions)))
            missions.extend(last_missions)

        # 整理并生成任务序列
        output_dir = None
        if appenv.context.output_dir:
            output_dir = Path(appenv.context.output_dir).resolve()
            appenv.say('输出目录将被替换为: "{}"'.format(output_dir))

        current_missions = asyncio.run(
            MissionMaker.auto_make_missions(
                self.presets,
                self.sources,
                external_output_dir=output_dir,
            )
        )
        if current_missions:
            appenv.say("生成了 {} 个任务。".format(len(current_missions)))
        missions.extend(current_missions)
        self._sort_and_set_missions(missions)

        # 生成脚本
        if appenv.context.save_script:
            maker = ScriptMaker(self.missions)
            maker.save(appenv.context.save_script)
            return

        # 执行转码任务
        if appenv.context.pretending_mode:
            appenv.say(
                "[dim]检测到[italic cyan underline]假装模式[/]，将不会真正执行任何操作。[/]"
            )

        mm = MissionMaster(self.missions, appenv.context.max_workers)
        asyncio.run(mm.run())



from rich.progress import Progress, TaskID

from cx_studio.utils import TextUtils


class ProgressTaskAgent:
    def __init__(self, progress: Progress | None = None, task_name: str | None = None):
        self._progress = progress
        self._task_name: str = task_name or TextUtils.random_string(5)
        self._task_id: TaskID | None = None

    @property
    def progress(self) -> Progress | None:
        return self._progress

    @property
    def task_name(self) -> str:
        return self._task_name

    @property
    def task_id(self) -> TaskID | None:
        return self._task_id

    async def __aenter__(self):
        if self._progress:
            self._task_id = self._progress.add_task(self._task_name)
        if self._progress and self._task_id:
            self._progress.update(self._task_id, visible=True)
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        if self._progress and self._task_id:
            self._progress.stop_task(self._task_id)
            self._progress.update(self._task_id, visible=False)
        self._task_id = None
        return False

    def __enter__(self):
        if self._progress:
            self._task_id = self._progress.add_task(self._task_name)
        if self._progress and self._task_id:
            self._progress.update(self._task_id, visible=True)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self._progress and self._task_id:
            self._progress.stop_task(self._task_id)
            self._progress.update(self._task_id, visible=False)
        self._task_id = None
        return False

    def start(self):
        if self._progress and self._task_id:
            self._progress.start_task(self._task_id)

    def stop(self):
        if self._progress and self._task_id:
            self._progress.stop_task(self._task_id)

    def update(
        self,
        *_args,
        total: float | None = None,
        completed: float | None = None,
        advance: float | None = None,
        description: str | None = None,
        visible: bool | None = None,
        refresh: bool = False,
        **fields: object,
    ):
        if self._progress and self._task_id:
            self._progress.update(
                self._task_id,
                *_args,
                total=total,
                completed=completed,
                advance=advance,
                description=description,
                visible=visible,
                refresh=refresh,
                **fields,
            )

    def advance(self, advance: float = 1.0):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, advance=advance)

    def set_total(self, total: float | None):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, total=total)

    def set_description(self, description: str):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, description=description)

    def set_completed(self, completed: float):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, completed=completed)

    def set_progress(self, completed: float, total: float | None):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, completed=completed, total=total)

    def show(self):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, visible=True)

    def hide(self):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, visible=False)

    def refresh(self):
        if self._progress and self._task_id:
            self._progress.update(self._task_id, refresh=True)



from collections.abc import Iterable
from .inspector_info import InspectorInfo
from .media_path_inspector import MediaPathInspector
from pathlib import Path, PurePath
import re


class EDLInspector(MediaPathInspector):
    FILENAME_PATTERN = re.compile(r"CLIP NAME: (.+)")

    def _is_inspectable(self, info: InspectorInfo) -> bool:
        if info.path.suffix != ".edl":
            return False
        if not info.is_decodable():
            return False
        return True

    def _inspect(self, info: InspectorInfo) -> Iterable[PurePath]:
        with open(info.path, "r", encoding=info.encoding) as fp:
            for line in fp:
                match = self.FILENAME_PATTERN.search(line)
                if match:
                    yield Path(match.group(1))



from typing import override
from cx_tools.app import IAppEnvironment, ConfigManager
from .appcontext import AppContext
from cx_wealth import rich_types as r
import signal
from collections.abc import Sequence
from pathlib import Path
import sys, os
from tempfile import TemporaryDirectory
import importlib.resources


class AppEnv(IAppEnvironment):
    def __init__(self):
        super().__init__()
        self.app_name = "HostsKeeper"
        self.app_version = "0.6.3.2"
        self.app_description = "根据配置文件更新 hosts"
        self.context = AppContext()

        self.config_manager = ConfigManager(self.app_name)
        self._temp_dir: TemporaryDirectory | None = None

    @override
    def start(self):
        self.show_banners()
        super().start()
        if self._temp_dir is None:
            self._temp_dir = TemporaryDirectory()
            appenv.whisper(f"临时目录已创建：{self.temp_dir}")

    @override
    def stop(self):
        if self._temp_dir is not None:
            self._temp_dir.cleanup()
            self._temp_dir = None
            appenv.whisper(f"临时目录已删除：{self.temp_dir}")
        super().stop()

    @property
    def temp_dir(self) -> Path:
        if self._temp_dir is None:
            self._temp_dir = TemporaryDirectory()
        return Path(self._temp_dir.name)

    @property
    def temp_hosts(self) -> Path:
        return self.temp_dir / "hosts"

    def show_banners(self):
        banners = []
        banner_text = importlib.resources.read_text(__package__, "banner.txt")
        banners.append(r.Align.center(banner_text))
        banners.append(r.Align.center("你的 hosts 由我来守护！"))
        banners.append(r.Align.center("v" + self.app_version))
        group = r.Group(*banners)
        appenv.console.print(group, style="bold cyan", highlight=False)

    def is_debug_mode_on(self):
        return self.context.debug_mode

    def is_pretending_mode_on(self):
        return self.context.pretending_mode

    def load_arguments(self, arguments: Sequence[str] | None = None):
        self.context = AppContext.from_arguments(arguments)

    @staticmethod
    def system_hosts_path() -> Path:
        system = sys.platform

        # Windows 系统（包括 win32, cygwin 等）
        if system.startswith("win"):
            # 优先使用 SYSTEMROOT 环境变量（Windows 标准）
            system_root = os.environ.get(
                "SYSTEMROOT", os.environ.get("WINDIR", "C:\\Windows")
            )
            return Path(system_root) / "System32" / "drivers" / "etc" / "hosts"

        # Unix-like 系统（Linux, macOS, FreeBSD, Solaris 等）
        elif system.startswith(("linux", "darwin", "freebsd", "sunos")):
            return Path("/etc/hosts")

        # 兜底：其他类 Unix 系统通常也使用 /etc/hosts
        elif "bsd" in system or "unix" in system.lower():
            return Path("/etc/hosts")

        # 无法识别的操作系统
        raise NotImplementedError(
            f"Unsupported operating system: {system}. "
            "Please provide the hosts file path manually."
        )


appenv = AppEnv()

# signal.signal(signal.SIGINT, appenv.handle_interrupt)



from collections.abc import AsyncGenerator, Iterable
from pathlib import Path
from .profile import Profile
import os, sys
from cx_studio.utils import EncodingUtils
import re
import asyncio
from .appenv import appenv
from cx_studio.utils import FunctionalUtils


class HostsBuilder:
    def __init__(self, hosts_file_path: Path | None = None, max_workers: int = -1):
        self.hosts_file_path = hosts_file_path or appenv.system_hosts_path()
        self.max_workers = (
            max_workers if max_workers > 0 else appenv.context.max_workers
        )
        self._semaphore = asyncio.Semaphore(self.max_workers)

    PROFILE_START_MARKER = re.compile(Profile.PROFILE_START_MARKER_PATTERN)
    PROFILE_END_MARKER = re.compile(Profile.PROFILE_END_MARKER_PATTERN)

    async def prepare_customed_lines(self) -> list[str]:
        async with self._semaphore:
            result = []
            encoding = EncodingUtils.detect_encoding(self.hosts_file_path)
            profile_entered: bool = False
            with self.hosts_file_path.open("r", encoding=encoding) as f:
                for line in f.readlines():
                    await asyncio.sleep(0)
                    if self.PROFILE_START_MARKER.match(line):
                        profile_entered = True
                        continue
                    if self.PROFILE_END_MARKER.match(line):
                        profile_entered = False
                        continue
                    if not profile_entered:
                        striped_line = line.strip()
                        if len(striped_line) > 0:
                            result.append(striped_line)
        return result

    async def prepare_profile_lines(self, profile: Profile) -> list[str]:
        async with self._semaphore:
            result = []
            async for line in profile.async_iter_lines():
                result.append(line)
        return result

    async def async_build_lines(self, profiles: Iterable[Profile]) -> list[list[str]]:
        tasks = [asyncio.create_task(self.prepare_customed_lines())]
        for profile in profiles:
            tasks.append(asyncio.create_task(self.prepare_profile_lines(profile)))
        return await asyncio.gather(*tasks)

    def iter_lines(self, profiles: Iterable[Profile]) -> Iterable[str]:
        collection = asyncio.run(self.async_build_lines(profiles))
        yield from FunctionalUtils.flatten_list(*collection)



import itertools
from pathlib import Path
import asyncio

from cx_studio.ffmpeg import FFmpegAsync, FFmpegArgumentsPreProcessor
from cx_studio.ffmpeg.cx_ff_infos import FFmpegCodingInfo
from cx_tools.app.safe_error import SafeError
from .appenv import appenv
from cx_wealth import IndexedListPanel


class Transcoder:
    def __init__(self, ffmpeg_executable: str | Path | None = None):
        self._ffmpeg = FFmpegAsync(ffmpeg_executable)
        self._taskID = None
        self._task_description = "Transcoding"
        self._ffmpeg_outputs = []

    def __enter__(self):
        self._taskID = appenv.progress.add_task(
            description="[green]准备中...[/]", total=None
        )
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._taskID is not None:
            appenv.progress.remove_task(self._taskID)
            self._taskID = None
        return False

    async def _on_verbose(self, line: str):
        """处理FFmpeg输出日志"""
        self._ffmpeg_outputs.append(line)

    async def _random_task_description(
        self, inputs: list[str] | None = None, outputs: list[str] | None = None
    ):
        """随机选择一个任务描述"""
        contents = []
        for x in inputs:
            contents.append(Path(x).name + " ->")
        for x in outputs:
            contents.append("-> " + Path(x).name)
        if not contents:
            contents.append("转码中……")

        for t in itertools.cycle(contents):
            if not self._ffmpeg.is_running():
                break
            self._task_description = t
            await asyncio.sleep(2)

    async def run(self, arguments: list[str] | None = None):
        """执行转码任务

        Args:
            arguments: FFmpeg命令行参数列表

        Returns:
            bool: 转码是否成功
        """
        arguments = arguments or []

        io_processor = FFmpegArgumentsPreProcessor(arguments)
        inputs = list(io_processor.iter_input_files())
        outputs = list(io_processor.iter_output_files())

        # 检查输入文件是否存在
        for i in inputs:
            if not Path(i).exists():
                raise SafeError(f"输入文件 {i} 不存在。")

        # 检查输出文件是否已存在
        existed_outputs = [x for x in outputs if Path(x).exists()]
        if existed_outputs and "-y" not in arguments:
            appenv.whisper(IndexedListPanel(existed_outputs, title="已存在的输出文件"))
            raise SafeError("请使用 -y 参数覆盖已存在的文件。")

        # 创建不存在的输出目录
        non_exist_dirs = filter(
            lambda x: not x.exists(), [Path(a).parent for a in outputs]
        )
        if non_exist_dirs:
            for d in non_exist_dirs:
                d.mkdir(parents=True, exist_ok=True)
            appenv.whisper(IndexedListPanel(non_exist_dirs, title="创建的输出目录"))

        # 统计文件数量用于显示
        m, n = len(inputs), len(outputs)
        summary = f"[blue][{m}->{n}][/]"

        # 设置状态更新监听器
        @self._ffmpeg.on("status_updated")
        def on_status_updated(status: FFmpegCodingInfo):
            current = status.current_time.total_seconds
            total = status.total_time.total_seconds if status.total_time else None

            # 格式化速度显示
            speed = "[bright_black][{:.2f}x][/]".format(status.current_speed)

            # 更新进度描述
            appenv.progress.update(
                self._taskID,
                completed=current,
                total=total,
                description=f"{summary}{speed}[green]{self._task_description}[/]",
            )

        @self._ffmpeg.on("started")
        def on_started():
            appenv.progress.update(
                self._taskID, description=f"{summary}[cx.success]开始转码...[/]"
            )

        @self._ffmpeg.on("finished")
        def on_finished():
            appenv.progress.update(
                self._taskID, description=f"{summary}[cx.success]转码完成[/]"
            )

        @self._ffmpeg.on("terminated")
        def on_terminated():
            appenv.progress.update(
                self._taskID, description=f"{summary}[cx.error]转码失败[/]"
            )
            appenv.whisper(self._ffmpeg_outputs, title="FFmpeg 输出")

        try:
            main_task = asyncio.create_task(self._ffmpeg.execute(arguments))
            desc_task = asyncio.create_task(
                self._random_task_description(inputs, outputs)
            )
            while not main_task.done():
                await asyncio.sleep(0.1)
                if appenv.really_wanna_quit_event.is_set():
                    appenv.whisper("尝试终止任务……")
                    self._ffmpeg.cancel()
                    await asyncio.sleep(1)
                    break
            result = main_task.result()
            await desc_task

        except asyncio.CancelledError:
            self._ffmpeg.cancel()
            result = False

        except SafeError:
            raise

        except Exception as e:
            appenv.progress.update(
                self._taskID, description=f"{summary}[cx.error]转码异常: {str(e)}[/]"
            )
            result = False
            raise SafeError(f"{str(e)}")
        finally:
            # 清理事件监听器
            self._ffmpeg.remove_all_listeners()

        if not result:
            if self._ffmpeg.is_canceled:
                raise SafeError("[cx.warning]用户取消了操作。[/]")
            else:
                raise SafeError("[cx.error]操作失败，请排查问题。[/]")

        return result



from operator import attrgetter
from typing import Literal
from collections.abc import Generator

from cx_wealth.wealth_label import WealthLabel
from media_killer.appenv import appenv
from .mission import Mission


class MissionArranger:
    def __init__(
        self,
        missions: list[Mission],
        sort_mode: Literal["source", "target", "preset", "x"] = "x",
    ):
        self.missions = missions
        self.sort_mode = sort_mode

        self._sorters = {
            "source": self.__sort_by_source,
            "target": self.__sort_by_target,
            "preset": self.__sort_by_preset,
            "x": self.__no_sort,
        }

    def __no_sort(self) -> Generator[Mission, None, None]:
        yield from self.missions

    def __sort_by_source(self) -> Generator[Mission, None, None]:
        yield from sorted(self.missions, key=attrgetter("source"))

    def __sort_by_target(self) -> Generator[Mission, None, None]:
        yield from sorted(self.missions, key=attrgetter("standard_target"))

    def __sort_by_preset(self) -> Generator[Mission, None, None]:
        yield from sorted(self.missions, key=lambda x: x.preset_id)

    def __iter__(self) -> Generator[Mission, None, None]:
        cache = set()
        for m in self._sorters[self.sort_mode]():
            if m in cache:
                appenv.say(WealthLabel(m), "是重复任务，已自动排除。")
                continue
            cache.add(m)
            yield m



from cx_studio.utils import TextUtils
from cx_wealth import WealthHelp
from cx_wealth import rich_types as r
import importlib.resources


class AppHelp(WealthHelp):
    def __init__(self):
        super().__init__(prog="hostskeeper")

        opt_group = self.add_group("命令和参数")

        opt_group.add_action(
            "COMMAND",
            name="命令",
            nargs="?",
            optional=True,
            metavar="list|show|edit|update|help",
            description=(
                """\
要执行的操作，可选值为：
 - [u]list[/]：列出所有配置文件。
 - [u]show[/]：显示当前配置文件的内容。
 - [u]edit[/]：编辑当前配置文件。
 - [u]update[/]：按照所有激活的配置文件更新hosts。
 - [u]help[/]：显示帮助信息。"""
            ),
        )

        opt_group.add_action(
            "PROFILE_ID",
            name="配置文件ID",
            optional=True,
            metavar="PROFILE_ID",
            description=("配置文件的ID，在 show 和 edit 命令中必须要指定。"),
        )

        opt_group.add_action(
            "--search-pattern",
            "-s",
            name="搜索模式",
            optional=True,
            metavar="SEARCH_PATTERN",
            description=(
                "如果在 list 中指定了搜索模式，将只会显示符合条件的配置文件。搜索模式支持 glob 形式的通配符，并会智能搜索配置文件的多种信息以找到符合条件的目标。"
            ),
        )

        opt_group.add_action(
            "--target",
            "--to",
            "-t",
            name="目标文件",
            optional=True,
            metavar="TARGET_HOSTS",
            description=("指定目标 hosts 文件，默认值为系统 hosts 文件。"),
        )

        misc_opts = self.add_group("杂项")
        misc_opts.add_action("-h", "--help", description="显示此帮助信息")
        misc_opts.add_action(
            "--tutorial", "--full-help", description="显示完整的教程内容"
        )
        misc_opts.add_action(
            "-d", "--debug", description="开启调试模式以观察更多的后台信息"
        )
        misc_opts.add_action(
            "-p",
            "--pretend",
            description="启用[bold blue]模拟运行模式[/]，不会进行任何文件操作。",
        )

        misc_opts.add_note(
            r.Text.from_markup(
                "[default dim italic]建议通过完整帮助信息学习更多使用技巧。[/]"
            )
        )

        self.description = TextUtils.unwrap(
            """本工具通过一系列配置文件自动编写操作系统的 hosts 文件。
            配置文件可通过 [u]list[/] [u]show[/] [u]edit[/] 等命令进行管理。
            执行 [u]update[/] 命令后，hosts 文件将被自动更新。
            当然，编辑 hosts 文件需要以管理员权限运行。"""
        )

        self.epilog = (
            "[link https://github.com/LambdaXIII/cx-studio-tk]Cxalio Studio Tools[/]"
        )

    @staticmethod
    def show_help(console: r.Console):
        console.print(AppHelp())

    @staticmethod
    def show_full_help(console: r.Console):
        md = importlib.resources.read_text(__package__, "help.md")
        content = r.Markdown(md, style="default")
        panel = r.Panel(
            content,
            title="Hosts Keeper 教程",
            width=90,
            style="bright_black",
            title_align="left",
        )
        console.print(r.Align.center(panel))



import re

from jpegger.simple_appcontext import SimpleAppContext
from .filters import (
    ImageFilterChain,
    AutoResizeFilter,
    AutoScaleFilter,
    ColorSpaceFilter,
)


class SimpleFilterChainBuilder:

    @staticmethod
    def __parse_size_str(size_str: str | None) -> tuple[int, int] | None:
        if size_str is None:
            return None

        size_str = size_str.strip()
        NUMBER_PAT = r"(\d+)[^\d]+(\d+)"
        match = re.match(NUMBER_PAT, size_str)
        if match:
            return int(match.group(1)), int(match.group(2))
        return None

    @staticmethod
    def build_filter_chain_from_simple_context(
        app_context: SimpleAppContext,
    ) -> ImageFilterChain:
        filters = []
        if app_context.scale_factor:
            filters.append(AutoScaleFilter(float(app_context.scale_factor)))
        else:
            iw, ih = SimpleFilterChainBuilder.__parse_size_str(app_context.size) or (
                None,
                None,
            )
            if app_context.width and not iw:
                iw = int(app_context.width)
            if app_context.height and not ih:
                ih = int(app_context.height)
            filters.append(AutoResizeFilter(iw, ih))

        if app_context.color_space:
            filters.append(ColorSpaceFilter(app_context.color_space))

        return ImageFilterChain(filters)



import subprocess
from .appenv import appenv
from pathlib import Path
from cx_studio.utils import SystemUtils, TextUtils
from datetime import datetime
import os, sys, shutil
from collections.abc import Iterable
from tempfile import TemporaryDirectory
from cx_studio.path_expander import CmdFinder


class HostsSaver:
    def __init__(
        self,
        target_hosts: Path | None = None,
        source_hosts: Path | Iterable[str] | None = None,
        pretending_mode: bool | None = None,
        backup_dir: Path | None = None,
    ):
        self.target_hosts = target_hosts or appenv.system_hosts_path()

        if source_hosts is None:
            self.source_hosts = appenv.temp_hosts
        elif isinstance(source_hosts, Path):
            self.source_hosts = source_hosts
        else:
            self.source_hosts = appenv.temp_dir / "customed_hosts_lines.txt"
            with self.source_hosts.open("w", encoding="utf-8") as f:
                f.writelines(source_hosts)

        self.pretending_mode = pretending_mode or appenv.context.pretending_mode
        self.backup_dir = backup_dir or (appenv.config_manager.config_dir / "backups")
        self._temp_dir: TemporaryDirectory | None = None

    @property
    def temp_dir(self) -> Path:
        if self._temp_dir is None:
            self._temp_dir = TemporaryDirectory()
        return Path(self._temp_dir.name)

    def __del__(self):
        if self._temp_dir is not None:
            self._temp_dir.cleanup()
            self._temp_dir = None

    def generate_backup_file_path(self) -> Path:
        name = (
            datetime.now().strftime("%Y%m%d_%H%M%S")
            + TextUtils.random_string(5)
            + ".bak"
        )
        return self.backup_dir / name

    def _generate_replace_script_pwsh(self, from_: Path, to_: Path) -> Path:
        script = self.temp_dir / "replace_hosts.ps1"
        with script.open("w", encoding="utf-8") as f:
            f.write(
                f'Copy-Item -Path "{from_.resolve()}" -Destination "{to_.resolve()}" -Force\n'
            )
        return script

    def _generate_replace_script_bash(self, from_: Path, to_: Path) -> Path:
        script = self.temp_dir / "replace_hosts.sh"
        with script.open("w", encoding="utf-8") as f:
            f.write(f'cp -f "{from_.resolve()}" "{to_.resolve()}"\n')
        return script

    def _generate_replace_script_auto(self, from_: Path, to_: Path) -> Path:
        if sys.platform.startswith("win"):
            return self._generate_replace_script_pwsh(from_, to_)
        else:
            return self._generate_replace_script_bash(from_, to_)

    def _run_replace_script(self, script: Path, as_admin: bool = False) -> bool:
        cmd = ["bash" if script.suffix == ".sh" else "pwsh"]
        cmd.append(script.resolve())

        if as_admin:
            appenv.whisper(f"目标文件无写入权限，正查找 sudo 命令...")
            sudo_cmd = CmdFinder().find("sudo")
            if sudo_cmd is None:
                appenv.say("[cx.error]sudo 命令未找到，将无法以管理员权限进行替换。")
                return False
            cmd.insert(0, sudo_cmd)

        try:
            subprocess.run(cmd, check=True)
            return True
        except subprocess.CalledProcessError as e:
            appenv.say(f"Replace script failed, error: {e}")
            return False

    def _backup_target_hosts(self, target_hosts: Path) -> bool:
        if not target_hosts.exists():
            appenv.whisper(
                f"Target hosts file {target_hosts} does not exist, skip backup."
            )
            return True
        backup_file = self.generate_backup_file_path()
        try:
            backup_file.parent.mkdir(parents=True, exist_ok=True)
            shutil.copy(target_hosts, backup_file)
        except Exception as e:
            appenv.say(f"无法创建备份文件，错误：{e}")
            return False
        return True

    def _show_hosts_lines(self, hosts_file: Path) -> None:
        with hosts_file.open("r", encoding="utf-8") as f:
            for line in f.readlines():
                print(line.strip())

    def save(self, target: Path | None = None) -> bool:
        """
        保存 hosts 文件。

        :param target: 目标 hosts 文件路径，默认值为系统 hosts 文件。
        :return: 如果成功保存新文件则返回 True，否则返回 False。
        """
        target = target or self.target_hosts
        if self.pretending_mode:
            appenv.say(f"[cx.info]假装模式已开启，新的内容将输出到标准输出。")
            self._show_hosts_lines(self.source_hosts)
            return False

        backup_result = self._backup_target_hosts(target)
        if not backup_result:
            appenv.say(
                f"[cx.warning]目标文件已存在且无法备份，将直接输出生成的 hosts 内容。"
            )
            return False

        script = self._generate_replace_script_auto(self.source_hosts, target)
        as_admin = SystemUtils.check_permission(target)
        run_result = self._run_replace_script(script, as_admin)
        if not run_result:
            appenv.say(f"[cx.warning]替换脚本执行失败，目标文件 {target} 未被修改。")
            self._show_hosts_lines(self.source_hosts)
            return False

        return True



from cx_tools.app import IAppEnvironment
from cx_tools.app.config_manager import ConfigManager


class AppEnv(IAppEnvironment):
    def __init__(self):
        super().__init__()
        self.app_name = "landlord"
        self.app_version = "0.1.0"
        self.config_manager = ConfigManager(self.app_name)


appenv = AppEnv()



from argparse import ArgumentParser
from collections.abc import Sequence

from cx_studio.utils import TextUtils
from cx_wealth import WealthHelp


class SimpleAppContext:
    def __init__(self, **kwargs):
        self.inputs: list[str] = []
        self.show_help: bool = False
        self.scale_factor: float | None = None
        self.size: str | None = None
        self.width: int | None = None
        self.height: int | None = None
        self.color_space: str | None = None
        self.format: str | None = None
        self.quality: int | None = None
        self.output_dir: str | None = None
        self.overwrite: bool = False
        self.debug_mode: bool = False

        for k, v in kwargs.items():
            if k in self.__dict__:
                self.__dict__[k] = v

    def __rich_repr__(self):
        yield from self.__dict__.items()

    @classmethod
    def from_arguments(cls, arguments: Sequence[str] | None = None):
        parser = cls.__make_parser()
        args = parser.parse_args(arguments)
        return cls(**vars(args))

    @staticmethod
    def __make_parser() -> ArgumentParser:
        parser = ArgumentParser(
            description="Jpegger 是一个简单的批量转换图片的命令行工具。", add_help=False
        )

        parser.add_argument("inputs", nargs="*")
        parser.add_argument(
            "--help", "-h", action="store_true", help="显示帮助信息", dest="show_help"
        )
        parser.add_argument("--scale", action="store", dest="scale_factor")
        parser.add_argument("--size", "-s", action="store", dest="size")
        parser.add_argument("--width", action="store", dest="width")
        parser.add_argument("--height", action="store", dest="height")
        parser.add_argument(
            "--color-space", "-c", choices=["RGB", "CMYK", "L"], dest="color_space"
        )
        parser.add_argument("--format", "-f", action="store", dest="format")
        parser.add_argument("--quality", "-q", action="store", dest="quality")
        parser.add_argument("--output", "-o", action="store", dest="output_dir")
        parser.add_argument(
            "--force-overwrite", "-y", action="store_true", dest="overwrite"
        )
        parser.add_argument("--debug", "-d", action="store_true", dest="debug_mode")

        return parser

    def __rich_detail__(self):
        ignore_text = "[red](忽略)[/red]"
        if self.scale_factor:
            yield "缩放因子", self.scale_factor
        if self.size:
            yield f"缩放尺寸{ignore_text if self.scale_factor else ""}", self.size
        if self.width:
            yield f"缩放宽度{ignore_text if self.scale_factor or self.size else ""}", self.width
        if self.height:
            yield f"缩放高度{ignore_text if self.scale_factor or self.size else ""}", self.height
        if self.color_space:
            yield "颜色空间", self.color_space
        if self.quality:
            yield "编码质量", self.quality
        if self.output_dir:
            yield "输出目录", self.output_dir
        if self.overwrite:
            yield "强制覆盖", self.overwrite
        if self.debug_mode:
            yield "调试模式", self.debug_mode

        known_keys = [
            "inputs",
            "show_help",
            "scale_factor",
            "size",
            "width",
            "height",
            "color_space",
            "format",
            "quality",
            "output_dir",
            "overwrite",
            "debug_mode",
        ]
        other_values = {k: v for k, v in self.__dict__.items() if k not in known_keys}

        yield from other_values.items()

        if self.inputs:
            yield "输入文件", self.inputs


class SimpleHelp(WealthHelp):
    def __init__(self):
        super().__init__(prog="jpegger")
        self.description = TextUtils.unwrap(
            """Jpegger是一个简单的批量转换图片的命令行工具。

            使用选项可以简单地控制输出图片的尺寸、编码质量和色彩空间。
            本工具旨在快速地进行简单的批量处理，所以暂不提供更高级的客制化功能。
            """
        )
        self.epilog = (
            "[link https://github.com/LambdaXIII/cx-studio-tk]Cxalio Studio Tools[/]"
        )

        basic_opts = self.add_group("基本选项")
        basic_opts.add_action(
            "inputs", nargs="+", metavar="FILE", description="需要转码的文件"
        )
        basic_opts.add_action(
            "-f",
            "--format",
            metavar="FORMAT",
            description="指定输出格式，默认沿用原始格式",
        )
        basic_opts.add_action(
            "-q",
            "--quality",
            metavar="QUALITY",
            description="指定输出质量，默认使用内置的常用质量设置",
        )
        basic_opts.add_action(
            "-o", "--output", metavar="DIR", description="输出目录，默认为当前目录"
        )

        image_controls = self.add_group("图片处理", "对图像进行处理")
        image_controls.add_action(
            "--scale", metavar="FACTOR", description="按比例缩放图片的尺寸"
        )
        image_controls.add_action(
            "-s",
            "--size",
            metavar="WIDTHxHEIGHT",
            description="指定图片的尺寸，接受包含两个数字的表达式",
        )
        image_controls.add_action(
            "--width",
            metavar="WIDTH",
            description="指定图片的宽度，如果未指定高度则保持原始图像比例",
        )
        image_controls.add_action(
            "--height",
            metavar="HEIGHT",
            description="指定图片的高度，如果未指定宽度则保持原始图像比例",
        )

        process_control = self.add_group("其它选项")
        process_control.add_action(
            "--overwrite",
            "-y",
            description="强制覆盖已存在的文件，未设置时将会自动重命名目标文件",
        )
        process_control.add_action("--debug", description="显示调试信息")
        process_control.add_action("-h", "--help", description="显示帮助信息")



from ..hostrecord import HostRecord
from ..contenter_base import AbstractContenter, ContenterBase
from box import Box
from typing import override, AsyncGenerator


class DirectContenter(AbstractContenter):
    """直接内容器"""

    SCHEMA: str = "direct_content"

    def __init__(
        self,
        package: Box | dict | None = None,
        profile_metadata: Box | dict | None = None,
        **kwargs
    ) -> None:
        super().__init__(package, profile_metadata, **kwargs)
        self.ip: str | None = self.package.get("ip") or None
        self.domains: list[str] = self.package.get("domains") or []
        self.comment: str | None = self.package.get("comment") or None

    @override
    async def iter_records(self) -> AsyncGenerator[HostRecord, None]:
        yield HostRecord(self.ip, self.domains, self.comment)


ContenterBase.register_contenter(DirectContenter)



import asyncio
from typing import override, AsyncGenerator
from ..hostrecord import HostRecord
from ..contenter_base import AbstractContenter, ContenterBase
from box import Box
from pathlib import Path


class LocalContenter(AbstractContenter):
    """本地内容器"""

    SCHEMA: str = "local_content"

    def __init__(
        self,
        package: Box | dict | None = None,
        profile_metadata: Box | dict | None = None,
        **kwargs
    ) -> None:
        super().__init__(package, profile_metadata, **kwargs)
        self.file: str = self.package.get("file")
        self.description: str | None = self.package.get("description")
        self.encoding: str = self.package.get("encoding") or "utf-8"

    @property
    def file_path(self) -> Path | None:
        """文件路径"""
        if self.file is None:
            return None

        path = Path(self.file)
        if path.is_absolute():
            return path

        profile_path = self.profile_path
        if profile_path is None:
            return path.resolve()

        return (profile_path / path).resolve()

    @override
    async def iter_records(self) -> AsyncGenerator[HostRecord, None]:
        """迭代记录"""
        file_path = self.file_path
        if file_path is None or not file_path.exists():
            return
        with open(file_path, "r", encoding=self.encoding) as f:
            for line in f.readlines():
                await asyncio.sleep(0)
                record = HostRecord.from_line(line.strip())
                yield record


ContenterBase.register_contenter(LocalContenter)



from .profile import Profile
from pathlib import Path
from .appenv import appenv
from cx_studio.path_expander import SuffixFinder
from cx_studio.utils import PathUtils
import os, subprocess
from collections.abc import Iterable
import fnmatch


class ProfileManager:
    # _FINDER = SuffixFinder(".toml")

    def __init__(self, profile_dir: Path | None = None):
        self.profile_dir = (
            profile_dir.resolve() if profile_dir else appenv.config_manager.config_dir
        )
        self.__profiles: dict[str, Profile] = {}
        self.__needs_refresh: bool = True

    def __find_profiles(self) -> Iterable[Path]:
        yield from self.profile_dir.glob("**/*.toml")

    @property
    def profiles(self) -> dict[str, Profile]:
        if self.__needs_refresh:
            self.__profiles.clear()
            for filename in self.__find_profiles():
                profile = Profile.load(filename)
                if profile is None:
                    appenv.whisper(f"[cx.error]{filename} 为非法配置文件，已跳过。")
                    continue
                if profile.id in self.__profiles:
                    old_profile = self.__profiles[profile.id]
                    appenv.whisper(
                        f"[cx.warning]配置文件 {profile.id} 已经由 {old_profile.path.name} 提供，将被 {profile.path.name} 覆盖。"
                    )
                self.__profiles[profile.id] = profile
            self.__needs_refresh = False
        return self.__profiles

    @property
    def enabled_profiles(self) -> Iterable[Profile]:
        for profile in self.profiles.values():
            if profile.enabled:
                yield profile

    @property
    def disabled_profiles(self) -> Iterable[Profile]:
        for profile in self.profiles.values():
            if not profile.enabled:
                yield profile

    @property
    def all_profiles(self) -> Iterable[Profile]:
        yield from self.profiles.values()

    def find_profile(self, search_pattern: str | None = None) -> Iterable[Profile]:
        for profile in self.all_profiles:
            if (
                not search_pattern
                or fnmatch.fnmatch(profile.id, search_pattern)
                or fnmatch.fnmatch(profile.name, search_pattern)
                or fnmatch.fnmatch(profile.description, search_pattern)
                or fnmatch.fnmatch(profile.path.name, search_pattern)
            ):
                yield profile

    def generate_profile_path(self, profile_id: str) -> Path:
        return self.profile_dir / Path(PathUtils.force_suffix(profile_id, ".toml"))

    def create_profile(self, profile_id: str, path: Path | None = None) -> Path | None:
        filename = path or self.generate_profile_path(profile_id)
        if filename.exists():
            return None
        Profile.create(profile_id, filename)
        self.__needs_refresh = True
        return filename



from abc import ABC, abstractmethod
from box import Box
from .hostrecord import HostRecord
from typing import AsyncGenerator

from pathlib import Path


class AbstractContenter(ABC):
    """
    内容器基类。
    根据SHEMA确定需要处理的段落。
    """

    SCHEMA: str = ""

    def __init_subclass__(cls) -> None:
        if cls.SCHEMA == "":
            raise TypeError(f"子类 {cls.__name__} 必须重定义SCHEMA常量")

    def __init__(
        self,
        package: Box | dict | None = None,
        profile_metadata: Box | dict | None = None,
        **kwargs,
    ) -> None:
        super().__init__()
        self.package = package if isinstance(package, Box) else Box(package)
        self.package.update(kwargs)
        self.profile_metadata = (
            profile_metadata
            if isinstance(profile_metadata, Box)
            else Box(profile_metadata)
        )

    @property
    def profile_path(self) -> Path | None:
        """配置文件路径"""
        if self.profile_metadata is None:
            return None
        p = self.profile_metadata.get("path")
        if p is None:
            return None
        return Path(p)

    @abstractmethod
    async def iter_records(self) -> AsyncGenerator[HostRecord, None]:
        pass


class ContenterBase:
    """内容器管理器"""

    CONTENTERS: dict[str, type[AbstractContenter]] = {}

    @staticmethod
    def register_contenter(cls: type[AbstractContenter]) -> None:
        """注册内容器"""
        ContenterBase.CONTENTERS[cls.SCHEMA] = cls

    @staticmethod
    def create_contenter(
        schema: str,
        package: Box | dict | None = None,
        profile_metadata: Box | dict | None = None,
        **kwargs,
    ) -> type[AbstractContenter] | None:
        """获取内容器"""
        return (
            ContenterBase.CONTENTERS.get(schema)(package, profile_metadata, **kwargs)
            if schema in ContenterBase.CONTENTERS
            else None
        )



import asyncio
from abc import ABC, abstractmethod

from rich.console import Console

from cx_studio.utils.tools import DoubleTrigger
from cx_studio.utils import SystemUtils
import cx_wealth.rich_types as r
from typing import Union

from rich.highlighter import RegexHighlighter

import os, sys, ctypes

DEFAULT_STYLES = {
    "cx.info": "blue",
    "cx.debug": "bright_black",
    "cx.warning": "yellow",
    "cx.error": "red",
    "cx.argument": "bold green1",
    "cx.success": "green1",
    "cx.whisper": "dim",
    "cx.number": "yellow",
    "cx.brackets": "magenta",
    "cx.quotes": "light_pink1",
    "cx.filepath": "bold cyan underline",
}
cx_default_theme = r.Theme(DEFAULT_STYLES)


class CxHighlighter(RegexHighlighter):
    base_style = "cx."
    highlights = [
        r"(?P<brackets>\(.*?\))",  # 括号括起的
        r"(?P<quotes>\".*?\"|\'.*?\')",  # 引号引用的
        r"(?P<filepath>[A-Za-z]:[\\/][^:*?\"<>|\n]*)",  # Windows 文件路径
        r"(?P<filepath>[\\/][^:*?\"<>|\n]*)",  # Unix 文件路径
        r"(?P<number>\d+(?:\.\d+)?)",  # 数字
        r"(?P<argument>\b--?[a-zA-Z0-9_\-]+\b)",  # 命令行参数
    ]


class IAppEnvironment(ABC):

    def __init__(self):
        self.app_name = ""
        self.app_version = ""
        self.highlighter = CxHighlighter()
        self.console_theme = cx_default_theme
        self.console = Console(
            stderr=True,
            theme=self.console_theme,
            highlighter=self.highlighter,  # 全局安装默认的Highlighter
            highlight=False,  # 屏蔽默认的Highlighter，将只在 say 方法中启用
        )

        self.wanna_quit_event = asyncio.Event()
        self.really_wanna_quit_event = asyncio.Event()

        self.interrupt_handler = DoubleTrigger()

        @self.interrupt_handler.on("first_triggered")
        def __when_wanna_quit():
            self.whisper("[cx.error]触发中断信号…[/]")
            # self.wanna_quit = True
            self.wanna_quit_event.set()

        @self.interrupt_handler.on("second_triggered")
        def __when_really_wanna_quit():
            self.whisper("[cx.error]检测到强制中断信号…[/]")
            # self.really_wanna_quit = True
            self.really_wanna_quit_event.set()

    def handle_interrupt(self, _sig, _frame):
        self.interrupt_handler.trigger()

    # @abstractmethod
    def is_debug_mode_on(self):
        return False

    # @abstractmethod
    def start(self):
        self.whisper(
            "{} v{} environment started.".format(self.app_name, self.app_version)
        )

    # @abstractmethod
    def stop(self):
        self.whisper(
            "{} v{} environment stopped.".format(self.app_name, self.app_version)
        )

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()
        self.whisper("Bye ~")
        return False

    def say(self, *args, **kwargs):
        # highlighted_args = (
        #     self.highlighter(x) if isinstance(x, Union[str, r.Text]) else x
        #     for x in args
        # )
        kwargs["highlight"] = True
        self.console.print(*args, **kwargs)

    def whisper(self, *args, **kwargs):
        if self.is_debug_mode_on():
            # highlighted_args = (self.highlighter(x) or x for x in args)
            kwargs["style"] = "dim"
            kwargs["highlight"] = False
            self.console.print(*args, **kwargs)

    @staticmethod
    def is_user_admin() -> bool:
        return SystemUtils.is_user_admin()



from collections.abc import Sequence
from datetime import datetime
import asyncio
from cx_studio.core.cx_time import CxTime
from cx_studio.ffmpeg import FFmpegArgumentsPreProcessor
from cx_tools.app import IApplication, SafeError
from cx_wealth.indexed_list_panel import IndexedListPanel
from .mk_help_info import MKHelp
from .appenv import appenv
from pathlib import Path
from cx_wealth import rich_types as r
from .transcoder import Transcoder
from .prober import Prober


class FFPrettyApp(IApplication):
    """
    FFPretty应用类 - FFmpeg的异步包装器，提供友好的用户界面

    这个类封装了FFmpeg的异步执行功能，包括：
    - 命令行参数解析和处理
    - 输入输出文件验证
    - 实时进度显示
    - 错误处理和用户反馈
    """

    def __init__(self, arguments: Sequence[str] | None = None):
        """
        初始化FFPretty应用

        Args:
            arguments: 命令行参数列表，如果为None则使用系统参数
        """
        super().__init__(arguments)
        self.arguments = []

        # 处理命令行参数，设置调试模式
        for a in self.sys_arguments:
            if a == "-d" or a == "--debug":
                # 启用调试模式
                appenv.debug_mode = True
            else:
                # 收集其他参数
                self.arguments.append(a)

        # 如果没有指定覆盖参数，默认使用 -n (不覆盖)
        if "-y" not in self.arguments and "-n" not in self.arguments:
            self.arguments.append("-n")

    def start(self):
        """
        启动应用

        初始化应用环境并记录开始时间
        """
        appenv.start()
        self.start_time = datetime.now()

    def stop(self):
        """
        停止应用

        清理应用环境，如果运行时间超过5秒则显示运行时间
        """
        appenv.stop()
        time_span = datetime.now() - self.start_time

        # 只有运行时间超过5秒才显示时间统计，避免频繁显示
        if time_span.total_seconds() > 5:
            appenv.say(
                "执行结束，用时[cx.number]{}[/]。".format(
                    CxTime.from_seconds(time_span.total_seconds()).pretty_string
                )
            )

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        上下文管理器退出处理

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常追踪信息

        Returns:
            bool: 是否成功处理异常
        """
        result = False
        if exc_type is None:
            # 正常退出，没有异常
            pass
        elif issubclass(exc_type, SafeError):
            # 处理安全错误，显示错误信息
            appenv.say("[cx.error]错误：{}[/]".format(exc_val))
            result = True

        # 无论如何都要停止应用
        self.stop()
        return result

    def run_transcode(self):
        """运行转码过程"""
        with Transcoder(appenv.ffmpeg_executable) as transcoder:
            return asyncio.run(transcoder.run(self.arguments))

    def run_probe(self, files: list[Path]):
        prober = Prober()
        for x in files:
            prober.probe(x)

    def run(self) -> bool:
        # 验证参数
        if not appenv.ffmpeg_executable:
            raise SafeError("当前环境中未找到 [cx.filepath]ffmpeg[/] 可执行文件。")

        if not self.arguments:
            raise SafeError("未提供任何参数。")

        if "-h" in self.arguments or "--help" in self.arguments:
            help_info = MKHelp()
            appenv.say(help_info)
            return True

        # 开始检查输入输出
        io_processor = FFmpegArgumentsPreProcessor(self.arguments)
        inputs = list(io_processor.iter_input_files())
        outputs = list(io_processor.iter_output_files())
        options = list(io_processor.iter_option_pairs())

        # 验证输入文件
        if (not inputs) and (not outputs):
            raise SafeError(
                "没有提供需要处理的文件，请按照 ffmpeg 的规则制定输入输出文件，或直接制定需要探测的文件。"
            )

        # 如果输入输出都有
        if len(inputs) > 0 and len(outputs) > 0:
            # 显示输入输出文件列表
            appenv.whisper(
                IndexedListPanel(inputs, title="输入文件"),
                IndexedListPanel(outputs, title="输出文件"),
            )
            self.run_transcode()
        elif not options:
            # 进入解析模式
            self.run_probe([Path(x) for x in inputs + outputs])
        else:
            appenv.say("[cx.error]参数无法解读。")



from typing import Literal

from PIL.Image import Image

from .image_filter import IImageFilter

__all__ = ["AutoResizeFilter", "AutoScaleFilter"]


def _auto_resize(image: Image, width: int, height: int) -> Image:
    iw, ih = image.size
    if (width, height) == (iw, ih):
        return image

    if iw <= ih:
        _factor = width / iw
        scaled_image = image.resize((width, int(ih * _factor)))
        y = (scaled_image.height - height) / 2
        return scaled_image.crop((0, y, width, y + height))
    else:
        _factor = height / ih
        scaled_image = image.resize((int(iw * _factor), height))
        x = (scaled_image.width - width) / 2
        return scaled_image.crop((x, 0, x + width, height))


class AutoResizeFilter(IImageFilter):
    """Auto resize filter
    Resize image to fit the given width and height.

    If `width` and `height` are both None, the image will be resized to a fixed size.
    If `width` or `height` is None, the resized image will keep the aspect ratio.
    If `width` and `height` are both not None, the image remains.

    **No matter how the resizing is done,
    the image will be cropped to fit the given size and positioned in the center.**

    Parameters
    width: int | None
        The width of the resized image.
    height: int | None
        The height of the resized image.
    """

    ResizingMode = Literal["fixed", "width_fixed", "height_fixed", "remains"]

    def __init__(self, width: int | None = None, height: int | None = None):
        super().__init__()
        self._width = None if width is None or width <= 0 else width
        self._height = None if height is None or height <= 0 else height

    @property
    def resizing_mode(self) -> ResizingMode:
        if self._width and self._height:
            return "fixed"
        elif self._width and not self._height:
            return "width_fixed"
        elif self._height and not self._width:
            return "height_fixed"
        return "remains"

    def get_target_size(self, image: Image) -> tuple[int, int]:
        target_width, target_height = iw, ih = image.size
        match (self.resizing_mode):
            case "fixed":
                target_width, target_height = self._width, self._height
            case "width_fixed":
                assert self._width
                target_width = self._width
                ratio = target_width / iw
                target_height = int(ih * ratio)
            case "height_fixed":
                assert self._height
                target_height = self._height
                ratio = target_height / ih
                target_width = int(iw * ratio)
        assert target_width
        assert target_height
        return target_width, target_height

    def run(self, image: Image) -> Image:
        iw, ih = self.get_target_size(image)
        return _auto_resize(image, iw, ih)

    def __rich_label__(self):
        na = "[red]N/A[/red]"
        yield super().__rich_label__()
        yield f"[blue]({self._width or na}:{self._height or na})[/]"

    def __filter_description__(self) -> str:
        match (self.resizing_mode):
            case "fixed":
                return f"调整图像分辨率至 {self._width}x{self._height}"
            case "width_fixed":
                return f"将图像宽度调整为 {self._width} ，并保持原图比例缩放高度"
            case "height_fixed":
                return f"将图像高度调整为 {self._height} ，并保持原图比例缩放宽度"
        return "不对图像做任何处理"


class AutoScaleFilter(IImageFilter):
    """AutoScaleFilter

    Same as AutoResizeFilter, but uses a factor instead of a size.
    ** If the size is 1.0 or negative, the imgae remains.**
    """

    def __init__(self, factor: float = 1.0):
        super().__init__()
        self.factor = factor

    def get_target_size(self, image: Image) -> tuple[int, int]:
        iw, ih = image.width, image.height
        if self.factor <= 0 or self.factor == 1.0:
            return iw, ih

        target_width = int(iw * self.factor)
        target_height = int(ih * self.factor)
        return target_width, target_height

    def run(self, image: Image) -> Image:
        iw, ih = self.get_target_size(image)
        return _auto_resize(image, iw, ih)

    def __rich_label__(self):
        yield from super().__rich_label__()
        yield f"[blue]({self.factor:.2f}x)[/]"

    def __filter_description__(self) -> str:
        if self.factor == 1.0 or self.factor <= 0:
            return "不对图像做任何处理"
        return f"将图像宽度和高度缩放 {self.factor:.2f} 倍"



# from . import profile_adapters

from .application import Application

def run():
    from rich.traceback import install
    install(show_locals=False, word_wrap=True, suppress=["rich"])
    with Application() as app:
        app.run()


from abc import ABC, abstractmethod

from PIL.Image import Image


class IImageFilter(ABC):
    @abstractmethod
    def run(self, image: Image) -> Image:
        pass

    def filter_name(self):
        return self.__class__.__name__.replace("Filter", "")

    @abstractmethod
    def __filter_description__(self) -> str:
        pass

    def __rich_label__(self):
        yield f"[yellow]{self.filter_name()}[/yellow]"



from typing import Any, Iterable
from cx_studio.core.cx_filesize import FileSize
from cx_studio.core.cx_time import CxTime
import cx_wealth.rich_types as r


class StreamInfo:
    def __init__(self, data: dict[str, Any]):
        self.data = data

    @property
    def stream_index(self) -> int:
        return self.data.get("index", -1)

    @property
    def codec_type(self) -> str:
        c_type = self.data.get("codec_type", None)
        return c_type

    @property
    def codec_name(self) -> str:
        return self.data.get("codec_name", None)

    @property
    def codec_long_names(self) -> Iterable[str]:
        c_long_name = self.data.get("codec_long_name", "")
        return c_long_name.split(" / ")

    def __rich_label__(self):
        index = self.stream_index
        s_type = self.codec_type
        match s_type:
            case "video":
                yield r.Text(f"视频流 #{index}", style="blue")
            case "audio":
                yield r.Text(f"音频流 #{index}", style="blue")
            case "subtitle":
                yield r.Text(f"字幕流 #{index}", style="blue")
            case _:
                yield r.Text(f"未知流 #{index}", style="blue")

        yield r.Text(self.codec_name, style="green1")
        profile = self.data.get("profile", None)
        if profile:
            yield r.Text(f"[{profile}]", style="green1")

        width, height = self.data.get("width", None), self.data.get("height", None)
        if width and height:
            yield r.Text(f"{width}x{height}", style="cx.number")

        sample_ratio = self.data.get("sample_aspect_ratio", None)
        if sample_ratio and sample_ratio != "1:1":
            yield r.Text(f"[{sample_ratio}]", style="cx.warning")
            cw, ch = self.data.get("coded_width", None), self.data.get(
                "coded_height", None
            )
            if cw and ch:
                yield r.Text(f"({cw}x{ch})", style="cx.info")

        if s_type == "video":
            frame_rate = self.data.get("avg_frame_rate", None)
            if frame_rate:
                yield r.Text(f"{frame_rate}fps", style="cx.whisper")

        if s_type == "audio":
            sample_rate = self.data.get("sample_rate", None)
            if sample_rate:
                yield r.Text(f"{sample_rate}Hz", style="cx.whisper")
            channel_layout = self.data.get("channel_layout", None)
            if channel_layout:
                yield r.Text(f"[{channel_layout}]", style="cx.info")

        duration_text = self.data.get("duration", None)
        if duration_text:
            duration = CxTime.from_seconds(float(duration_text))
            yield r.Text(f"[{duration.to_timestamp()}]", style="cx.number")

        bitrate_text = self.data.get("bit_rate", None)
        if bitrate_text:
            bitrate = FileSize.from_bytes(int(bitrate_text))
            yield r.Text(f"{bitrate.pretty_string}/s", style="cx.error")



from .inspector_info import *
from .media_path_inspector import *
from .resolve_metadata_inspector import *
from .edl_inspector import *
from .legacy_xml_inspector import *
from .fcpxml_inspector import *
from .inspector_chain import *
from .filelist_inspector import *



import asyncio
from typing import override, AsyncGenerator
import urllib.request
from ..hostrecord import HostRecord
from ..contenter_base import AbstractContenter, ContenterBase
from box import Box
from pathlib import Path


class UrlContenter(AbstractContenter):
    """URL内容器"""

    SCHEMA: str = "url_content"

    def __init__(
        self,
        package: Box | dict | None = None,
        profile_metadata: Box | dict | None = None,
        **kwargs
    ) -> None:
        super().__init__(package, profile_metadata, **kwargs)
        self.url: str | None = self.package.get("url")
        self.description: str | None = self.package.get("description")
        self.encoding: str = self.package.get("encoding") or "utf-8"

    def get_content(self) -> str:
        """获取URL内容"""
        # TODO:改进为异步获取
        if self.url is None:
            return ""
        with urllib.request.urlopen(self.url) as response:
            return response.read().decode(self.encoding)

    @override
    async def iter_records(self) -> AsyncGenerator[HostRecord, None]:
        """迭代记录"""
        content = self.get_content()
        for line in content.splitlines():
            await asyncio.sleep(0)
            yield HostRecord.from_line(line.strip())


ContenterBase.register_contenter(UrlContenter)
